@@ -160,7 +160,6 @@ NULL
 #' @examples
 #' ht <- hux(a = 1:3, b = 4:6, add_colnames = TRUE)
 #' where(ht > 2)
-#' where(is_a_number(ht))
 #'
 where <- function(expr) which(expr, arr.ind = TRUE)
 
@@ -176,6 +175,7 @@ NULL
 # which causes R CMD check to throw a wobbly
 is_a_number <- function(x) {
   if (is.data.frame(x)) {
+    if (nrow(x) == 0) return(matrix(FALSE, 0, ncol(x)))
     res <- sapply(x, is_a_number)
     dim(res) <- dim(x)
     return(res)
